@@ -31,7 +31,7 @@ from atlas.runtime.agent_loop.tool_loop import ToolCallAgentGraphState
 from atlas.prompts import RewrittenStudentPrompts
 from atlas.types import Plan
 from atlas.types import Step
-from atlas.learning.playbook import resolve_playbook
+from atlas.learning.playbook import extract_few_shot_examples, resolve_playbook
 from atlas.learning.usage import get_tracker
 
 logger = logging.getLogger(__name__)
@@ -180,7 +180,12 @@ class Student:
         context.metadata["active_actor"] = "student"
         prompt = self._compose_planner_prompt(task)
         try:
-            response = await self._adapter.ainvoke(prompt, metadata={"mode": "planning"})
+            # Pass structured task payload to adapters for BYOA integrations
+            adapter_metadata = {
+                "mode": "planning",
+                "task_payload": task,  # Original task before prompt composition
+            }
+            response = await self._adapter.ainvoke(prompt, metadata=adapter_metadata)
             self._apply_usage_payload(getattr(response, "usage", None))
             response = self._unwrap_adapter_payload(response)
             if isinstance(response, (dict, list)):
@@ -234,12 +239,25 @@ class Student:
         execution_context = ExecutionContext.get()
         execution_context.metadata["active_actor"] = "student"
         execution_context.metadata["_reasoning_origin"] = ("student", "execution")
+
+        # Pass step payload via LangGraph config to avoid race conditions in concurrent execution
+        # Each step gets its own config, preventing the last-writer-wins issue with shared metadata
+        step_payload = {
+            "step_id": step.id,
+            "description": step.description,
+            "depends_on": step.depends_on,
+        }
+
         state = ToolCallAgentGraphState(messages=messages)
         final_messages: Sequence[BaseMessage] | None = None
         step_manager = execution_context.intermediate_step_manager
         llm_snapshots: List[Any] = []
         llm_text_chunks: List[str] = []
-        async for event in graph.astream_events(state, config={"recursion_limit": recursion_limit}, version="v2"):
+        run_config = {
+            "recursion_limit": recursion_limit,
+            "metadata": {"step_payload": step_payload},
+        }
+        async for event in graph.astream_events(state, config=run_config, version="v2"):
             self._handle_stream_event(step, event, step_manager)
             chunk_messages = self._extract_messages_from_graph_payload(event.get("data", {}).get("chunk"))
             if chunk_messages:
@@ -473,9 +491,47 @@ class Student:
 
     def _compose_system_prompt(self, base_prompt: str, label: str) -> str:
         playbook, _, metadata = resolve_playbook("student", apply=self._apply_learning_prompts)
+        try:
+            context = ExecutionContext.get()
+            inject_mode = context.metadata.get("learning_injection_mode", "prefix")
+            inject_examples = bool(context.metadata.get("learning_inject_few_shot", False))
+            token_budget = int(context.metadata.get("learning_few_shot_token_budget", 0) or 0)
+            redaction_patterns = context.metadata.get("learning_redaction_patterns") or []
+            chars_per_token = float(context.metadata.get("learning_chars_per_token", 3.5))
+            max_entries = int(context.metadata.get("learning_max_entries", 10))
+            max_examples_per_block = int(context.metadata.get("learning_max_examples_per_block", 2))
+        except Exception:
+            inject_mode = "prefix"
+            inject_examples = False
+            token_budget = 0
+            redaction_patterns = []
+            chars_per_token = 3.5
+            max_entries = 10
+            max_examples_per_block = 2
+
+        if inject_examples and playbook:
+            examples_block = extract_few_shot_examples(
+                metadata,
+                "student",
+                max_tokens=token_budget,
+                redaction_patterns=redaction_patterns,
+                chars_per_token=chars_per_token,
+                max_entries=max_entries,
+                max_examples_per_block=max_examples_per_block,
+            )
+            if examples_block:
+                playbook = f"{playbook}\n\n{examples_block}"
+
         block = self._format_playbook_block(label, playbook, metadata, role="student")
         base = base_prompt.strip()
-        segments = [segment for segment in (block, base) if segment]
+
+        # Injection mode: prefix (playbook first) or suffix (playbook after base prompt for KV cache)
+        if inject_mode == "suffix":
+            segments = [base, block]
+        else:  # prefix (default)
+            segments = [block, base]
+
+        segments = [segment for segment in segments if segment]
         return "\n\n".join(segments) if segments else ""
 
     def _record_student_cue_hits(self, text: str, step_id: int) -> None:
