@@ -3,7 +3,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, field, asdict
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Union
 
 
 @dataclass
@@ -121,6 +121,8 @@ class AtlasStepTrace:
     metadata: Dict[str, Any] = field(default_factory=dict)
     artifacts: Dict[str, Any] = field(default_factory=dict)
     deliverable: Any | None = None
+    runtime: Optional[Dict[str, Any]] = None
+    depends_on: Optional[List[Union[int, str]]] = None
 
     def to_dict(self) -> Dict[str, Any]:
         return {
@@ -138,8 +140,15 @@ class AtlasStepTrace:
             "metadata": self.metadata,
             "artifacts": self.artifacts,
             "deliverable": self.deliverable,
+            "runtime": self.runtime,
+            "depends_on": self.depends_on,
         }
 
+    @property
+    def attempt_history(self) -> Optional[List[Dict[str, Any]]]:
+        """Get attempt history from metadata."""
+        return self.metadata.get("attempt_history")
+
 
 @dataclass
 class AtlasSessionTrace:
@@ -150,6 +159,12 @@ class AtlasSessionTrace:
     plan: Dict[str, Any]
     steps: List[AtlasStepTrace]
     session_metadata: Dict[str, Any] = field(default_factory=dict)
+    session_reward: Optional[Dict[str, Any]] = None
+    trajectory_events: Optional[List[Dict[str, Any]]] = None
+    student_learning: Optional[str] = None
+    teacher_learning: Optional[str] = None
+    learning_history: Optional[Dict[str, Any]] = None
+    adaptive_summary: Optional[Dict[str, Any]] = None
 
     def to_dict(self) -> Dict[str, Any]:
         return {
@@ -158,4 +173,50 @@ class AtlasSessionTrace:
             "plan": self.plan,
             "steps": [step.to_dict() for step in self.steps],
             "session_metadata": self.session_metadata,
+            "session_reward": self.session_reward,
+            "trajectory_events": self.trajectory_events,
+            "student_learning": self.student_learning,
+            "teacher_learning": self.teacher_learning,
+            "learning_history": self.learning_history,
+            "adaptive_summary": self.adaptive_summary,
         }
+
+    @property
+    def learning_key(self) -> Optional[str]:
+        """Get learning key from session metadata."""
+        return self.session_metadata.get("learning_key")
+
+    @property
+    def teacher_notes(self) -> Optional[List[Any]]:
+        """Get teacher notes from session metadata."""
+        return self.session_metadata.get("teacher_notes")
+
+    @property
+    def reward_summary(self) -> Optional[Dict[str, Any]]:
+        """Get reward summary from session metadata."""
+        return self.session_metadata.get("reward_summary")
+
+    @property
+    def drift(self) -> Optional[Dict[str, Any]]:
+        """Get drift detection results from session metadata."""
+        return self.session_metadata.get("drift")
+
+    @property
+    def drift_alert(self) -> Optional[Any]:
+        """Get drift alert flag from session metadata."""
+        drift_payload = self.session_metadata.get("drift")
+        if drift_payload is None:
+            return self.session_metadata.get("drift_alert")
+        if isinstance(drift_payload, dict):
+            return drift_payload.get("drift_alert") or self.session_metadata.get("drift_alert")
+        return self.session_metadata.get("drift_alert")
+
+    @property
+    def triage_dossier(self) -> Optional[Dict[str, Any]]:
+        """Get triage dossier from session metadata."""
+        return self.session_metadata.get("triage_dossier")
+
+    @property
+    def reward_audit(self) -> Optional[List[Dict[str, Any]]]:
+        """Get reward audit trail from session metadata."""
+        return self.session_metadata.get("reward_audit")
