@@ -8,6 +8,7 @@ from typing import Any, Dict, Optional, TextIO, Tuple
 from atlas.runtime.models import IntermediateStep
 from atlas.runtime.models import IntermediateStepType
 from atlas.runtime.orchestration.execution_context import ExecutionContext
+from atlas.utils.reactive.subscription import Subscription
 from atlas.types import Result
 
 
@@ -15,7 +16,7 @@ class ConsoleTelemetryStreamer:
     def __init__(self, output: TextIO | None = None) -> None:
         self._output = output or sys.stdout
         self._lock = threading.Lock()
-        self._subscription = None
+        self._subscription: Subscription[IntermediateStep] | None = None
         self._execution_context: ExecutionContext | None = None
         self._task_name = ""
         self._session_started_at: datetime.datetime | None = None
@@ -147,22 +148,41 @@ class ConsoleTelemetryStreamer:
             )
         self._attempt_starts.pop((step_id, attempt), None)
 
-        reward_payload = self._coerce_dict(evaluation.get("reward"))
-        score_value = reward_payload.get("score")
-        score_text = f"{float(score_value):.2f}" if isinstance(score_value, (int, float)) else "n/a"
-        judges = reward_payload.get("judges")
-        rim_scores: list[str] = []
-        if isinstance(judges, list):
-            for index, judge in enumerate(judges, start=1):
-                judge_payload = self._coerce_dict(judge)
-                identifier = judge_payload.get("identifier") or f"judge{index}"
-                judge_score = judge_payload.get("score")
-                if isinstance(judge_score, (int, float)):
-                    rim_scores.append(f"{identifier}:{float(judge_score):.2f}")
-        rim_display = ", ".join(rim_scores) if rim_scores else "none"
-        self._write(
-            f"STEP {step_id}: retry {attempt} | Reward score={score_text} | RIM scores: {rim_display}"
-        )
+        reward_skipped = bool(runtime_payload.get("reward_skipped"))
+        if not reward_skipped:
+            reward_payload = self._coerce_dict(evaluation.get("reward"))
+            score_value = reward_payload.get("score")
+            score_text = f"{float(score_value):.2f}" if isinstance(score_value, (int, float)) else "n/a"
+            judges = reward_payload.get("judges")
+            judge_scores: list[str] = []
+            rationale_snippet: str | None = None
+            if isinstance(judges, list):
+                for index, judge in enumerate(judges, start=1):
+                    judge_payload = self._coerce_dict(judge)
+                    identifier = judge_payload.get("identifier") or f"judge{index}"
+                    judge_score = judge_payload.get("score")
+                    if isinstance(judge_score, (int, float)):
+                        judge_scores.append(f"{identifier}:{float(judge_score):.2f}")
+                    if rationale_snippet is None:
+                        judge_rationale = judge_payload.get("rationale")
+                        if isinstance(judge_rationale, str) and judge_rationale.strip():
+                            rationale_snippet = self._shorten(judge_rationale.strip(), 160)
+            if rationale_snippet is None:
+                reward_rationale = reward_payload.get("rationale")
+                if isinstance(reward_rationale, str) and reward_rationale.strip():
+                    rationale_snippet = self._shorten(reward_rationale.strip(), 160)
+            judge_display = ", ".join(judge_scores) if judge_scores else "none"
+            self._write(
+                f"STEP {step_id}: retry {attempt} | Reward score={score_text} | Judge scores: {judge_display}"
+            )
+            if rationale_snippet:
+                self._write(
+                    f"STEP {step_id}: retry {attempt} | Reward rationale: {rationale_snippet}"
+                )
+        else:
+            self._write(
+                f"STEP {step_id}: retry {attempt} | Reward evaluation deferred to session-level judge"
+            )
 
     def _capture_plan_metadata(self) -> None:
         if self._execution_context is None:
@@ -223,9 +243,6 @@ class ConsoleTelemetryStreamer:
         confidence = adaptive_summary.get("confidence") if isinstance(adaptive_summary, dict) else None
         if isinstance(confidence, (int, float)):
             summary_line += f" | adaptive_confidence={confidence:.2f}"
-        certification_flag = adaptive_summary.get("certification_run") if isinstance(adaptive_summary, dict) else None
-        if certification_flag:
-            summary_line += " | certification=True"
         self._write(summary_line)
         if attempt_summary:
             self._write(f"  attempts: {attempt_summary}")
@@ -260,15 +277,12 @@ class ConsoleTelemetryStreamer:
             return
         mode = adaptive.get("adaptive_mode")
         confidence = adaptive.get("confidence")
-        certification = adaptive.get("certification_run")
         probe = adaptive.get("probe") if isinstance(adaptive.get("probe"), dict) else None
         line_parts = ["Adaptive:"]
         if mode:
             line_parts.append(f"mode={mode}")
         if confidence is not None:
             line_parts.append(f"confidence={confidence}")
-        if certification:
-            line_parts.append("certification=True")
         self._write(" ".join(line_parts))
         if probe:
             evidence = probe.get("evidence") if isinstance(probe.get("evidence"), list) else []
@@ -292,6 +306,15 @@ class ConsoleTelemetryStreamer:
             score = reward_payload.get("score")
             if isinstance(score, (int, float)):
                 rationale = reward_payload.get("rationale")
+                judges = reward_payload.get("judges")
+                if (not isinstance(rationale, str) or not rationale.strip()) and isinstance(judges, list):
+                    for judge in judges:
+                        if not isinstance(judge, dict):
+                            continue
+                        alt = judge.get("rationale")
+                        if isinstance(alt, str) and alt.strip():
+                            rationale = alt
+                            break
                 message = f"Reward score={score:.2f}"
                 if isinstance(rationale, str) and rationale.strip():
                     message += f" ({self._shorten(rationale.strip(), 80)})"
@@ -311,21 +334,35 @@ class ConsoleTelemetryStreamer:
 
     def _collect_learning_highlights(self, metadata: Dict[str, Any]) -> list[tuple[str, str]]:
         highlights: list[tuple[str, str]] = []
-        student_learning = metadata.get("student_learning")
+        applied = metadata.get("learning_state")
+        if isinstance(applied, dict):
+            applied_student = applied.get("student_learning")
+            if isinstance(applied_student, str) and applied_student.strip():
+                highlights.append(("applied_student_learning", applied_student.strip()))
+            applied_teacher = applied.get("teacher_learning")
+            if isinstance(applied_teacher, str) and applied_teacher.strip():
+                highlights.append(("applied_teacher_learning", applied_teacher.strip()))
+        student_learning = metadata.get("session_student_learning")
         if isinstance(student_learning, str) and student_learning.strip():
-            highlights.append(("student_learning", student_learning.strip()))
-        teacher_learning = metadata.get("teacher_learning")
+            highlights.append(("new_student_learning", student_learning.strip()))
+        teacher_learning = metadata.get("session_teacher_learning")
         if isinstance(teacher_learning, str) and teacher_learning.strip():
-            highlights.append(("teacher_learning", teacher_learning.strip()))
+            highlights.append(("new_teacher_learning", teacher_learning.strip()))
         return highlights
 
     def _compute_metrics(self, result: Result) -> Tuple[str, int]:
         attempt_counts: dict[int, int] = {}
         for step_result in result.step_results:
             attempt_counts[step_result.step_id] = step_result.attempts
-        steps_meta = {}
+        steps_meta: dict[int, dict[str, Any]] = {}
         if self._execution_context is not None:
-            steps_meta = self._execution_context.metadata.get("steps", {}) or {}
+            raw_steps = self._execution_context.metadata.get("steps", {}) or {}
+            if isinstance(raw_steps, dict):
+                steps_meta = {
+                    step_id: meta
+                    for step_id, meta in raw_steps.items()
+                    if isinstance(step_id, int) and isinstance(meta, dict)
+                }
         if not attempt_counts and steps_meta:
             for step_id, meta in steps_meta.items():
                 attempts = len(meta.get("attempts", []))
